import os
from typing import Optional

import requests
from requests.adapters import HTTPAdapter
from urllib3 import Retry
import jfrog_ml
from jfrog_ml._log_config import logger


class HTTPClient:

    def __init__(self, auth, session: Optional[requests.Session] = None):
        self.auth = auth
        # add default headers
        if session is None:
            self.session = self._create_session()
        self._add_default_headers()
        self.timeout = os.getenv("JFML_TIMEOUT", default=10)

    def _create_session(self):
        session = requests.Session()
        adapter = HTTPAdapter(
            max_retries=RetryWithLog(total=5, backoff_factor=0.5, status_forcelist=[500, 502, 503, 504]))
        session.mount("http://", adapter)
        session.mount("https://", adapter)
        return session

    def post(self, url, data=None, params=None):
        return self.session.post(url, auth=self.auth, timeout=self.timeout, data=data, params=params)

    def get(self, url, params=None, stream=False):
        return self.session.get(url, auth=self.auth, params=params, stream=stream)

    def put(self, url, payload=None, files=None, stream=False, headers=None):
        return self.session.request(method="PUT", url=url, data=payload, auth=self.auth, files=files, stream=stream,
                                    timeout=self.timeout, headers=headers)

    def _add_default_headers(self):
        self.session.headers.update({'User-Agent': 'frogml-sdk-python/{}'.format(jfrog_ml.__version__)})


class RetryWithLog(Retry):
    """
     Adding extra logs before making a retry request
    """

    def __init__(self, *args, **kwargs):
        history = kwargs.get("history")
        if history is not None:
            logger.debug(f'Error: ${history[-1].error}\nretrying...')
        super().__init__(*args, **kwargs)
